﻿//*************************************************************************************************
//【サンプルプログラム】『Cyber Eye Version1.1』
//　Kinectのデプスイメージ画像を音情報に変換する
//【作成者】私設研究所Neo-Tech-Lab.com  上田智章
//【作成日】2013年3月9日 【改訂】2013年3月16日
//【注意事項】Kinect for Windows SDK Version1.6
//*************************************************************************************************
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing;         //【Bitmap】【Color】
using System.Drawing.Imaging; //【BitmapData】
using Microsoft.Kinect;       //【参照設定】プロジェクト(P)⇒参照の追加(F)⇒.NETタブ⇒Microsoft.Kinectを選択
using NTL.KinectLib;          //【NTL製Kinectライブラリ】
using Microsoft.DirectX.AudioVideoPlayback;

namespace Kinect_CyberEye2
{
    public struct WaveSound
    {
        public Audio[] AudioOutput; //【ダブルバッファ】
        public int[] DepthValue;    //【深度スキャンライン】【固定サイズ】
        public int VolumeR;         //【右側音量】0～32767
        public int VolumeL;         //【左側音量】0～32767
    }

    public partial class Form1 : Form
    {
        //【KINECT関連】
        private KinectSensor KS_KinectSensor; //【KINECTセンサ】★利用可能なKinectセンサ
        private Boolean KS_fReverse;          //【左右反転フラグ】true: 反転　false: 非反転
        private int KS_HeadAngle = 0;         //【Kinectヘッドの仰角を設定】設定範囲は±27°
        private ImageCamera DepthCamera = new ImageCamera();
        private CharpAudio DepthAudio = new CharpAudio();
        private const int maxDistance = 3000; //【最大距離】3000mm
        private const int minDistance = 500;  //【最小距離】500mm
        private const int nScan = 5; //【スキャンラインの本数】
        private WaveSound[] WaveOut = new WaveSound[nScan];
        private int[] Volume = new int[]
        { // 左側音量, 右側音量 
                    0,    32000,
                10000,    32000,
                32000,    32000,
                32000,    10000,
                32000,        0
        }; 
        private int Count = 0;  //【タイミング管理カウンタ】
        private int ACount = 0; //【ダブルバッファ管理用】0 or 1


        public Form1()
        {
            InitializeComponent();
            pictureBox1.SizeMode = PictureBoxSizeMode.StretchImage; //【画像をpictureBox1枠内にぴったりはめ込む】
            pictureBox2.SizeMode = PictureBoxSizeMode.StretchImage; //【画像がpictureBox2枠内にぴったりはめ込む】
            pictureBox3.SizeMode = PictureBoxSizeMode.StretchImage; //【画像がpictureBox3枠内にぴったりはめ込む】
            pictureBox4.SizeMode = PictureBoxSizeMode.StretchImage; //【画像がpictureBox4枠内にぴったりはめ込む】

            //***********************************************************
            //【Kinect検出】【共通】
            //***********************************************************
            KS_KinectSensor = DepthCamera.DetectKinect(); //【利用可能なKINECTセンサを取得する】
            //【検出結果の表示】
            if (KS_KinectSensor == null) //【Kinect接続が確認できなかった場合】
            {
                pictureBox2.Image = imageList1.Images[1];
                MessageBox.Show(
                   "接続可能なKINECTセンサが見当たりません。",
                   "KINECT for Windows Error Message",
                   MessageBoxButtons.OK
                );
                return;
            }
            else //【Kinect検出に成功】
            {
                pictureBox2.Image = imageList1.Images[0];
            }

            //***********************************************************
            //【Kinectの初期化処理】
            //***********************************************************
            KS_fReverse = checkBox1.Checked; //【チェックされていたら左右反転】
            DepthCamera.Initialize(true, false, true, false, KS_HeadAngle);
            pictureBox3.Image = DepthCamera.DepthBarImage;
            KS_KinectSensor.ColorFrameReady += new EventHandler<ColorImageFrameReadyEventArgs>(ColorFrameReady);
            KS_KinectSensor.DepthFrameReady += new EventHandler<DepthImageFrameReadyEventArgs>(DepthImageReady); //【DepthImageReadyをcallするよう設定】
            this.FormClosing += new FormClosingEventHandler(Form1_FormClosing);

            DepthAudio.Initialize();
            for (int i = 0; i < nScan; i++)
            {
                WaveOut[i].AudioOutput = new Audio[2];  //【ダブルバッファ】
                WaveOut[i].DepthValue = new int[KS_KinectSensor.DepthStream.FrameHeight];   //【サイズ固定】
                WaveOut[i].VolumeL = Volume[2 * i];     //【左側音量】
                WaveOut[i].VolumeR = Volume[2 * i + 1]; //【右側音量】
            }
        }

        //***************************************************************
        //【終了要求ボタン】
        //***************************************************************
        void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            DepthCamera.Finish();
            if (KS_KinectSensor != null) { KS_KinectSensor.Dispose(); }
        }

        //***************************************************************
        //【上下角変更ボタン】KINECTの上下角を変更。±27°の範囲内。時間を置かないとエラーが発生する
        //***************************************************************
        private void button2_Click(object sender, EventArgs e)
        {
            int angle = int.Parse(textBox1.Text); //【設定角度を読み込む】±27°
            DepthCamera.SetElevationAngle(angle);
        }

        //***************************************************************
        //【表示画像の左右反転指定フラグ】
        //***************************************************************
        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            KS_fReverse = !KS_fReverse; //【表示の左右反転】true: 反転　false: 非反転
        }


        //***************************************************************
        //【ColorImageの取得イベント】
        //***************************************************************
        void ColorFrameReady(object sender, ColorImageFrameReadyEventArgs e)
        {
            using (ColorImageFrame f = e.OpenColorImageFrame())
            {
                if (f != null)
                {
                    f.CopyPixelDataTo(DepthCamera.ImageDataXRGB);
                    pictureBox4.Image = DepthCamera.TransferColorImage(KS_fReverse);
                }
            }
        }

        //***************************************************************
        //【DepthImageの取得イベント】
        //***************************************************************
        private void DepthImageReady(object sender, DepthImageFrameReadyEventArgs e)
        {
            using (DepthImageFrame f = e.OpenDepthImageFrame())
            {
                if (f != null)
                {
                    int i, j;
                    string filename;
                    f.CopyDepthImagePixelDataTo(DepthCamera.DepthImage); //【KINECTのDepth情報を配列DepthImageにコピー】
                    pictureBox1.Image = DepthCamera.ConvertToDepthImage(KS_fReverse);  //【pictureBox1に画像表示】

                    //【白い補助線と５本のスキャンラインと測定位置を描画する】
                    Graphics g = Graphics.FromImage(pictureBox1.Image);
                    g.DrawLine(new Pen(Color.White, 3.0f), 0, 240, 640, 240); //【横白線】
                    for (i = 0; i < nScan; i++)
                    {
                        g.DrawLine(new Pen(Color.Red, 3.0f), i * 100 + 120, 0, i * 100 + 120, 480); //【スキャンライン】
                        g.DrawRectangle(new Pen(Color.GreenYellow, 5.0f), i * 100 + 115, (f.Height - 1) - 8 * Count, 10, 10); //【測定位置】
                    }
                    g.Dispose();
                    Count++;
                    if (Count == 60)
                    {
                        Count = 0;
                        //【2秒間に1回】
                        Boolean flagAlert = false;
                        for (i = 0; i < nScan; i++)
                        {
                            for (j = 0; j < f.Height; j++)
                            {
                                WaveOut[i].DepthValue[j] = DepthCamera.DepthImage[i * 100 + 120 + 640 * (f.Height - 1 - j)].Depth;
                                if (WaveOut[i].DepthValue[j] > 3000)
                                {
                                    flagAlert = true;
                                }
                            }
                            //【警告音】maxDistance以上離れたの画素が存在した場合に警告音を鳴らす
                            if (flagAlert)
                            {
                         //       Audio AlertAudio = new Audio("../../Wave/NG.wav");
                                Audio AlertAudio = new Audio("Wave/NG.wav");
                                AlertAudio.Play();
                            }
                        }
                        //【waveファイルの作成と再生開始】
                        for (i = 0; i < nScan; i++)
                        {
                            filename = "Work" + "B" + String.Format("{0:D1}", ACount) + "L" + String.Format("{0:D1}", i) + ".wav";
                            DepthAudio.CreateWave(filename,              //【waveファイル名称】
                                                  WaveOut[i].DepthValue, //【スキャンラインi上の深度情報】
                                                  WaveOut[i].VolumeL,    //【左側音量】
                                                  WaveOut[i].VolumeR,    //【右側音量】
                                                  minDistance,           //【最小距離】
                                                  maxDistance);          //【最大距離】
                            WaveOut[i].AudioOutput[ACount] = new Audio(filename); //【waveファイルを読込む】
                            WaveOut[i].AudioOutput[ACount].Play();      //【読込んだwaveファイルを再生開始】
                        }
                        //【バッファ切替】
                        ACount++; 
                        if (ACount == 2)
                        {
                            ACount = 0;
                        }
                        //【裏バッファ側の音の再生を停止終了】
                        for (i = 0; i < nScan; i++)
                        {
                            if (WaveOut[i].AudioOutput[ACount] != null)
                            {
                                WaveOut[i].AudioOutput[ACount].Stop();
                                WaveOut[i].AudioOutput[ACount].Dispose();
                            }
                        }
                    }
                    //【0.5秒毎に直交３軸加速度センサの値を表示する】
                    if ((Count == 0) || (Count == 15) || (Count == 30) || (Count == 45)) 
                    {
                        label1.Text = "X = " + DepthCamera.AccelerometerCurrentValue.X.ToString() + " [g]";
                        label2.Text = "Y = " + DepthCamera.AccelerometerCurrentValue.Y.ToString() + " [g]";
                        label7.Text = "Z = " + DepthCamera.AccelerometerCurrentValue.Z.ToString() + " [g]";
                        label8.Text = "|Acc| = " + DepthCamera.AccelerometerCurrentValue.X.ToString() + " [g]";
                    }
                }
                else
                {
                    //【DepthFrameがnullだったのは要求が間に合わなかったと考えられます】
                }
            }
        }
    }
}
